package com.zfire.jiasm.syncdata.download.worker;


import com.zfire.jiasm.syncdata.constant.TaskConfigEnum;
import com.zfire.jiasm.syncdata.service.GLPPWXGQueryAddService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 格力品牌网点维修工 证件照，服务类目，新增同步 初始化进来
 *     后期不做新增同步，只做修改同步
 * 扫描下载表
 * 1：平台本地无格力总部的师傅数据不处理
 * 2：师傅已有证件信息不处理-这里无论是什么证件都不再新增证件数据
 *
 * 网点服务类目信息同步
 * 1：平台无师傅信息不处理
 * 2：师傅在平台 师傅网点对应品牌网点信息表中不存在不处理（不存在找不到对应的平台网点信息） worker_brand_account：
 */
@Slf4j
@Component
public class GLPPWXGQueryAdd {
    @Value("${wxgwzc}")
    private String wxgwzc = "";
    @Value("${lmxxc}")
    private String lmxxc = "";
    @Value("${zjzxxc}")
    private String zjzxxc = "";
    @Value("${wxgwztc}")
    private String wxgwztc = "";

    @Resource
    GLPPWXGQueryAddService glppwxgQueryAddService;


    @Scheduled(fixedDelayString = "${zjzxx}")
    public void process() throws Exception {

        wxgQueryAddCertInfo();

        wxgQueryAddCategory();

        wxgQueryAddWXGInfo();

    }




    /**
     * 从格力总部同步师傅的证件照 只用来新增
     * 师傅状态离职不同步
     * 师傅在平台中不存在不同步
     * 已同步过的不再同步
     */
    public void wxgQueryAddCertInfo() throws Exception {
        if (zjzxxc.equals(TaskConfigEnum.FALSE.getCode())){
            return;
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        log.info("开始同步师傅证件照信息: \uF0B7{}",sdf.format(new Date()));

        glppwxgQueryAddService.extractedCertInfo();

        log.info("同步师傅证件照信息结束: \uF0B7{}",sdf.format(new Date()));
    }


    /**
     * 新增 师傅服务类目信息 这里只做新增(只新增从格力总部过来的师傅服务的类目信息)
     */
    public void wxgQueryAddCategory() throws Exception {
        if (lmxxc.equals(TaskConfigEnum.FALSE.getCode())){
            return;
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        log.info("开始同步网点师傅服务类目信息: \uF0B7{}",sdf.format(new Date()));

        glppwxgQueryAddService.extractedAddCategory();

        log.info("同步网点师傅服务类目信息结束: \uF0B7{}",sdf.format(new Date()));
    }


    /**
     * 新增师傅地理位置  不再执行 由 wxgQueryAddWXGInfo 来完成
     */
    public void wxgQueryAdd() throws Exception {
        if (wxgwzc.equals(TaskConfigEnum.FALSE.getCode())){
            return;
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        log.info("开始同步网点师傅地理位置信息: \uF0B7{}",sdf.format(new Date()));

        glppwxgQueryAddService.extractedwxgQueryAdd();

        log.info("同步网点师傅地理位置信息结束: \uF0B7{}",sdf.format(new Date()));
    }


    /**
     * 新增格力网点维修工数据
     */
    public void wxgQueryAddWXGInfo() throws Exception {
        if (wxgwztc.equals(TaskConfigEnum.FALSE.getCode())){
            return;
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        log.info("开始同步网点维修工数据信息: \uF0B7{}",sdf.format(new Date()));

        glppwxgQueryAddService.extracteWxgQueryAddWXGInfo();

        log.info("同步网点维修工数据信息结束: \uF0B7{}",sdf.format(new Date()));
    }

}
